'use strict';

const path = require('path');
const through = require('through2');
const gutil = require('gulp-util');
const mcss = require('mcss');
const replaceExt = require('replace-ext');
const PluginError = gutil.PluginError;

module.exports = function (opt) {
    let program = through.obj((file, enc, cb) => {
        if (file.isNull())
            return cb(null, file);
        if (file.isStream())
            return cb(new PluginError('gulp_mcss', 'Streaming not supported'));

        const options = Object.assign({
            filename: file.path
        }, opt);

        let include;
        if (options.include) {
            if(!Array.isArray(options.include)) options.include = [options.include];
            include = options.include.map(function(p) {
                return path.resolve(process.cwd(), p);
            });
        }

        try {
            let instance = mcss(options);

            if (include) {
                instance.include(include);
            }

            mcss(options).translate().done((text) => {
                file.contents = new Buffer(text);
                file.path = replaceExt(file.path, '.css');
                cb(null, file);
            }).fail((err) => {
                mcss.error.format(err);
                console.log(err.message);
                cb();
            }).always(() => {
                /**
                 * 得到当前mcss的依赖
                 * @type {[type]}
                 */
                program.emit('returnDependencys', Object.keys(instance.get('imports')));
            });
        } catch (err) {
            cb(new PluginError('gulp_mcss', err));
        }
    });
    return program;
};